/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_OBJECT_H
#define DPC_IMPL_TYPE_OBJECT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "impl_type_float.h"
#include "impl_type_uint8.h"
#include "impl_type_boolean.h"
#include "dpc/impl_type_vector_objectintention.h"

namespace dpc {
struct Object {
    ::UInt32 object_id{ 0U };
    ::Float object_exist_probability{ 0.0F };
    ::UInt8 classification{ 0U };
    ::Float relative_center_x{ 0.0F };
    ::Float relative_center_y{ 0.0F };
    ::Float relative_position_x{ 0.0F };
    ::Float relative_position_y{ 0.0F };
    ::Float absolute_velocity_x{ 0.0F };
    ::Float absolute_velocity_y{ 0.0F };
    ::Float absolute_acceleration_x{ 0.0F };
    ::Float absolute_acceleration_y{ 0.0F };
    ::Float absolute_yawrate{ 0.0F };
    ::Float size_width{ 0.0F };
    ::Float size_length{ 0.0F };
    ::Boolean intentions_valid{ false };
    ::UInt8 intentions_number{ 0U };
    ::dpc::vector_ObjectIntention intention_list;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(object_id);
        fun(object_exist_probability);
        fun(classification);
        fun(relative_center_x);
        fun(relative_center_y);
        fun(relative_position_x);
        fun(relative_position_y);
        fun(absolute_velocity_x);
        fun(absolute_velocity_y);
        fun(absolute_acceleration_x);
        fun(absolute_acceleration_y);
        fun(absolute_yawrate);
        fun(size_width);
        fun(size_length);
        fun(intentions_valid);
        fun(intentions_number);
        fun(intention_list);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(object_id);
        fun(object_exist_probability);
        fun(classification);
        fun(relative_center_x);
        fun(relative_center_y);
        fun(relative_position_x);
        fun(relative_position_y);
        fun(absolute_velocity_x);
        fun(absolute_velocity_y);
        fun(absolute_acceleration_x);
        fun(absolute_acceleration_y);
        fun(absolute_yawrate);
        fun(size_width);
        fun(size_length);
        fun(intentions_valid);
        fun(intentions_number);
        fun(intention_list);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("object_id", object_id);
        fun("object_exist_probability", object_exist_probability);
        fun("classification", classification);
        fun("relative_center_x", relative_center_x);
        fun("relative_center_y", relative_center_y);
        fun("relative_position_x", relative_position_x);
        fun("relative_position_y", relative_position_y);
        fun("absolute_velocity_x", absolute_velocity_x);
        fun("absolute_velocity_y", absolute_velocity_y);
        fun("absolute_acceleration_x", absolute_acceleration_x);
        fun("absolute_acceleration_y", absolute_acceleration_y);
        fun("absolute_yawrate", absolute_yawrate);
        fun("size_width", size_width);
        fun("size_length", size_length);
        fun("intentions_valid", intentions_valid);
        fun("intentions_number", intentions_number);
        fun("intention_list", intention_list);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("object_id", object_id);
        fun("object_exist_probability", object_exist_probability);
        fun("classification", classification);
        fun("relative_center_x", relative_center_x);
        fun("relative_center_y", relative_center_y);
        fun("relative_position_x", relative_position_x);
        fun("relative_position_y", relative_position_y);
        fun("absolute_velocity_x", absolute_velocity_x);
        fun("absolute_velocity_y", absolute_velocity_y);
        fun("absolute_acceleration_x", absolute_acceleration_x);
        fun("absolute_acceleration_y", absolute_acceleration_y);
        fun("absolute_yawrate", absolute_yawrate);
        fun("size_width", size_width);
        fun("size_length", size_length);
        fun("intentions_valid", intentions_valid);
        fun("intentions_number", intentions_number);
        fun("intention_list", intention_list);
    }

    friend bool operator==(const ::dpc::Object& lhs, const ::dpc::Object& rhs) noexcept
    {
        return (lhs.object_id == rhs.object_id) && (fabs(static_cast<double>(lhs.object_exist_probability - rhs.object_exist_probability)) < DBL_EPSILON) && (lhs.classification == rhs.classification) && (fabs(static_cast<double>(lhs.relative_center_x - rhs.relative_center_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.relative_center_y - rhs.relative_center_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.relative_position_x - rhs.relative_position_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.relative_position_y - rhs.relative_position_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_velocity_x - rhs.absolute_velocity_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_velocity_y - rhs.absolute_velocity_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_acceleration_x - rhs.absolute_acceleration_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_acceleration_y - rhs.absolute_acceleration_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_yawrate - rhs.absolute_yawrate)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.size_width - rhs.size_width)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.size_length - rhs.size_length)) < DBL_EPSILON) && (lhs.intentions_valid == rhs.intentions_valid) && (lhs.intentions_number == rhs.intentions_number) && (lhs.intention_list == rhs.intention_list);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_OBJECT_H
